<?php

namespace Ivoz\Provider\Domain\Model\ResidentialDevice;

use Ivoz\Api\Core\Annotation\AttributeDefinition;
use Ivoz\Kam\Domain\Model\UsersLocation\RegistrationStatus;

class ResidentialDeviceDto extends ResidentialDeviceDtoAbstract
{
    /**
     * @deprecated as status it's already being served in collection responses
     */
    public const CONTEXT_STATUS = 'status';

    /**
     * @var RegistrationStatus[]
     * @AttributeDefinition(
     *     type="array",
     *     class="Ivoz\Kam\Domain\Model\UsersLocation\RegistrationStatus",
     *     description="Registration status"
     * )
     */
    private $status = [];

    /**
     * @var string
     * @AttributeDefinition(
     *     type="string",
     *     description="Registration domain"
     * )
     */
    private $domainName;

    public function addStatus(RegistrationStatus $status): static
    {
        $this->status[] = $status;

        return $this;
    }

    public function setDomainName(string $name): void
    {
        $this->domainName = $name;
    }

    public function toArray(bool $hideSensitiveData = false): array
    {
        $response = parent::toArray($hideSensitiveData);
        $response['domainName'] = $this->domainName;
        $response['status'] = array_map(
            function (RegistrationStatus $registrationStatus): array {
                return $registrationStatus->toArray();
            },
            $this->status
        );

        return $response;
    }

    /**
     * @inheritdoc
     * @codeCoverageIgnore
     */
    public static function getPropertyMap(string $context = '', string $role = null): array
    {
        if ($context === self::CONTEXT_STATUS) {
            $baseAttributes = [
                'id' => 'id',
                'name' => 'name',
                'domainName' => 'domainName',
                'directConnectivity' => 'directConnectivity',
                'status' => [[
                    'contact',
                    'received',
                    'publicReceived',
                    'expires',
                    'userAgent'
                ]]
            ];

            if ($role === 'ROLE_BRAND_ADMIN') {
                $baseAttributes['companyId'] = 'company';
            }

            $response = $baseAttributes;
        } elseif ($context === self::CONTEXT_COLLECTION) {
            $response = [
                'id' => 'id',
                'name' => 'name',
                'description' => 'description',
                'domainId' => 'domain',
                'directConnectivity' => 'directConnectivity',
                'status' => [[
                    'contact',
                    'received',
                    'publicReceived',
                    'expires',
                    'userAgent'
                ]]
            ];

            if ($role === 'ROLE_BRAND_ADMIN') {
                $response['companyId'] = 'company';
                $response['rtpEncryption'] = 'rtpEncryption';
                $response['multiContact'] = 'multiContact';
            }
        } else {
            $response = parent::getPropertyMap(...func_get_args());

            if ($context === self::CONTEXT_DETAILED) {
                $response['status'] = [[
                        'contact',
                        'received',
                        'publicReceived',
                        'expires',
                        'userAgent'
                ]];
            }

            unset($response['domainId']);
        }

        if ($role === 'ROLE_BRAND_ADMIN') {
            return self::filterFieldsForBrandAdmin($response);
        }

        if ($role === 'ROLE_COMPANY_ADMIN') {
            return self::filterFieldsForCompanyAdmin($response);
        }

        return $response;
    }

    public function denormalize(array $data, string $context, string $role = ''): void
    {
        $contextProperties = self::getPropertyMap($context, $role);
        if ($role === 'ROLE_BRAND_ADMIN') {
            $contextProperties['brandId'] = 'brand';
            unset($contextProperties['multiContact']);
        } elseif ($role === 'ROLE_COMPANY_ADMIN') {
            $contextProperties['companyId'] = 'company';
            unset($contextProperties['directConnectivity']);
        }

        $this->setByContext(
            $contextProperties,
            $data
        );
    }

    /**
     * @param array $response
     * @return array
     */
    private static function filterFieldsForBrandAdmin(array $response): array
    {
        $allowedFields = [
            'name',
            'description',
            'transport',
            'ip',
            'port',
            'password',
            'allow',
            'fromDomain',
            'directConnectivity',
            'ddiIn',
            'maxCalls',
            't38Passthrough',
            'id',
            'companyId',
            'transformationRuleSetId',
            'outgoingDdiId',
            'languageId',
            'companyId',
            'rtpEncryption',
            'multiContact',
            'domainName',
            'status',
            'domainId',
            'ruriDomain',
            'proxyUserId'
        ];

        return array_filter(
            $response,
            function ($key) use ($allowedFields): bool {
                return in_array($key, $allowedFields, true);
            },
            ARRAY_FILTER_USE_KEY
        );
    }

    /**
     * @param array $response
     * @return array
     */
    private static function filterFieldsForCompanyAdmin(array $response): array
    {
        $allowedFields = [
            'id',
            'name',
            'description',
            'password',
            'directConnectivity',
            'transport',
            'ip',
            'port',
            'multiContact',
            'languageId',
            'transformationRuleSetId',
            'outgoingDdiId',
            'allow',
            'fromDomain',
            'ddiIn',
            't38Passthrough',
            'maxCalls',
            'rtpEncryption',
            'domainName',
            'status',
            'domainId',
            'ruriDomain',
            'trustSDP'
        ];

        return array_filter(
            $response,
            function ($key) use ($allowedFields): bool {
                return in_array($key, $allowedFields, true);
            },
            ARRAY_FILTER_USE_KEY
        );
    }
}
